"""
This module is an example of a barebones QWidget plugin for napari

It implements the Widget specification.
see: https://napari.org/plugins/guides.html?#widgets

Replace code below according to your needs.
"""
from typing import TYPE_CHECKING

from magicgui import magic_factory
from qtpy.QtWidgets import QHBoxLayout, QPushButton, QWidget

if TYPE_CHECKING:
    import napari


class ExampleQWidget(QWidget):
    # your QWidget.__init__ can optionally request the napari viewer instance
    # in one of two ways:
    # 1. use a parameter called `napari_viewer`, as done here
    # 2. use a type annotation of 'napari.viewer.Viewer' for any parameter
    def __init__(self, napari_viewer):
        super().__init__()
        self.viewer = napari_viewer

        btn = QPushButton("Click me!")
        btn.clicked.connect(self._on_click)

        self.setLayout(QHBoxLayout())
        self.layout().addWidget(btn)

    def _on_click(self):
        unit = self.viewer.scale_bar.unit
        if unit == "um":
            self.viewer.scale_bar.unit = ""
            for layer in self.viewer.layers:
                layer.scale = [1, 1, 1]
                self.viewer.reset_view()
        else:
            self.viewer.scale_bar.unit = "um"
            for layer in self.viewer.layers:
                layer.scale = [1, 0.5, 0.5]
                self.viewer.reset_view()

        print("napari has", len(self.viewer.layers), "layers")


@magic_factory
def example_magic_widget(img_layer: "napari.layers.Image"):
    print(f"you have selected {img_layer}")


# Uses the `autogenerate: true` flag in the plugin manifest
# to indicate it should be wrapped as a magicgui to autogenerate
# a widget.
def example_function_widget(img_layer: "napari.layers.Image"):
    print(f"you have selected {img_layer}")



"""
This module is an example of a barebones sample data provider for napari.

It implements the "sample data" specification.
see: https://napari.org/plugins/guides.html?#sample-data

Replace code below according to your needs.
"""
from __future__ import annotations

import numpy


def make_sample_data():
    """Generates an image"""
    return numpy.random.rand(512, 512)



import numpy as np
from astropy.visualization import PercentileInterval


def get_display_range(arr, percentile=99):
    p = PercentileInterval(percentile)
    p_limits = {
        c: p.get_limits(arr.sel(channel=c).data.ravel().clip(0, 1000000))
        for c in arr.channel.values
    }
    p1 = np.array([*[p[0] for p in p_limits.values()]])[..., None, None]
    p2 = np.array([*[p[1] for p in p_limits.values()]])[..., None, None]
    return p1, p2



"""
This module is an example of a barebones numpy reader plugin for napari.

It implements the Reader specification, but your plugin may choose to
implement multiple readers or even other plugin contributions. see:
https://napari.org/plugins/guides.html?#readers
"""
from contextlib import suppress
from pathlib import Path

import dask.array as da
import xarray as xr
import zarr

from .utils import get_display_range


def get_store_from_path(path):
    if path.startswith("s3://"):
        from imaxt_image.io.s3 import get_s3_store

        _, alias, instrument, name = Path(path).parts
        store = None
        for extra in ["mos", "mos.zarr"]:  # v0.9 of pipeline
            with suppress(PermissionError):
                store = get_s3_store(name, alias, instrument, extra)
            if store is not None:
                break
    else:
        path = Path(path)
        if (path / "mos").exists():
            store = path / "mos"
        else:
            store = path / "mos.zarr"  # v0.9 of pipeline
    return store


def napari_get_reader(path):
    """A basic implementation of a Reader contribution.

    Parameters
    ----------
    path : str or list of str
        Path to file, or list of paths.

    Returns
    -------
    function or None
        If the path is a recognized format, return a function that accepts the
        same path or list of paths, and returns a list of layer data tuples.
    """
    if isinstance(path, list):
        # A path list does not make sense for this plugin
        return None

    store = get_store_from_path(path)

    try:
        zarr.open(store, mode="r")
    except zarr.errors.PathNotFoundError:
        return None

    # otherwise we return the *function* that can read ``path``.
    return reader_function


def reader_function(path):
    """Take a path or list of paths and return a list of LayerData tuples.

    Readers are expected to return data as a list of tuples, where each tuple
    is (data, [add_kwargs, [layer_type]]), "add_kwargs" and "layer_type" are
    both optional.

    Parameters
    ----------
    path : str or list of str
        Path to file, or list of paths.

    Returns
    -------
    layer_data : list of tuples
        A list of LayerData tuples where each tuple in the list contains
        (data, metadata, layer_type), where data is a numpy array, metadata is
        a dict of keyword arguments for the corresponding viewer.add_* method
        in napari, and layer_type is a lower-case string naming the type of
        layer. Both "meta", and "layer_type" are optional. napari will
        default to layer_type=="image" if not provided
    """
    store = get_store_from_path(path)
    ds = xr.open_zarr(store, group="l.16")
    channels = list(ds.channel.values)

    cube = []
    ds = xr.open_zarr(store, group="l.16")
    bscale = 1
    bzero = 0
    p1, p2 = get_display_range(ds["S001"].sel(z=0) * bscale + bzero)
    for group in ["", "l.2", "l.4", "l.8", "l.16"]:
        ds = xr.open_zarr(store, group=group)
        if "type" in ds.coords:  # v0.9 of pipeline
            ds = ds.sel(type="mosaic").astype("float32")
        data = da.stack(
            [
                (ds[s].sel(z=0).data.astype("float32") * bscale + bzero - p1)
                / (p2 - p1)
                * 10000
                for s in list(ds)
            ]
        )
        cube.append(data)

    # optional kwargs for the corresponding viewer.add_* method
    add_kwargs = {
        "multiscale": True,
        "name": [f"C{ch}" for ch in channels],
        "scale": [1, 1, 1],
        "colormap": ["red", "green", "blue", "cyan"],
        "blending": "ADDITIVE",
        "channel_axis": 1,
        "contrast_limits": [0, 30000],
    }

    layer_type = "image"  # optional, default is "image"
    return [(cube, add_kwargs, layer_type)]



__version__ = "0.1.4"

from ._reader import napari_get_reader
from ._sample_data import make_sample_data
from ._widget import ExampleQWidget, example_magic_widget
from ._writer import write_multiple, write_single_image

__all__ = (
    "napari_get_reader",
    "write_single_image",
    "write_multiple",
    "make_sample_data",
    "ExampleQWidget",
    "example_magic_widget",
)
